@@ -2,10 +2,6 @@
 #include <setjmp.h>
 #include "libfastmint.h"
 
-//#if defined(WIN32) && !defined(__MMX__)
-//#define __MMX__
-//#endif
-
 #if (defined(__i386__) || defined(__AMD64__)) && defined(__GNUC__) && defined(__MMX__)
 typedef int mmx_d_t __attribute__ ((mode(V2SI)));
 typedef int mmx_q_t __attribute__ ((mode(DI)));
@@ -288,7 +284,7 @@ static inline mmx_d_t S(int n, mmx_d_t X)
 #endif
 #define MINTER_CALLBACK_CLEANUP_FP __builtin_ia32_emms()
 
-unsigned long minter_mmx_standard_1(int bits, int* best, char *block, const uInt32 IV[5], int tailIndex, unsigned long maxIter, MINTER_CALLBACK_ARGS)
+unsigned long minter_mmx_standard_1(int bits, int* best, unsigned char *block, const uInt32 IV[5], int tailIndex, unsigned long maxIter, MINTER_CALLBACK_ARGS)
 {
 #if (defined(__i386__) || defined(__AMD64__)) && defined(__GNUC__) && defined(__MMX__)
   MINTER_CALLBACK_VARS;
@@ -308,10 +304,6 @@ unsigned long minter_mmx_standard_1(int bits, int* best, char *block, const uInt
   unsigned char *X = (unsigned char*) W;
   unsigned char *output = (unsigned char*) block;
 	
-  /* Make sure we don't get into an infinite loop */
-  if(maxIter > 0xFFFFFFF0U)
-    maxIter = 0xFFFFFFF0U;
-	
   *best = 0;
 
   /* Splat Kn constants into MMX-style array */
@@ -345,10 +337,10 @@ unsigned long minter_mmx_standard_1(int bits, int* best, char *block, const uInt
   for(t=0; t < 5; t++) {
     Hw[t*2+0] = Hw[t*2+1] =
     pHw[t*2+0] = pHw[t*2+1] = IV[t];
-	}
+  }
 	
   /* The Tight Loop - everything in here should be extra efficient */
-  for(iters=0; iters < maxIter; iters += 2) {
+  for(iters=0; iters < maxIter-2; iters += 2) {
 
     /* Encode iteration count into tail */
     /* Iteration count is always 2-aligned, so only least-significant character needs multiple lookup */
@@ -356,16 +348,26 @@ unsigned long minter_mmx_standard_1(int bits, int* best, char *block, const uInt
     X[(((tailIndex - 1) & ~3) << 1) + (((tailIndex - 1) & 3) ^ 3) +  0] = p[(iters & 0x3e) + 0];
     X[(((tailIndex - 1) & ~3) << 1) + (((tailIndex - 1) & 3) ^ 3) +  4] = p[(iters & 0x3e) + 1];
     if(!(iters & 0x3f)) {
-      X[(((tailIndex - 2) & ~3) << 1) + (((tailIndex - 2) & 3) ^ 3) +  0] =
+      if ( iters >> 6 ) {
+	X[(((tailIndex - 2) & ~3) << 1) + (((tailIndex - 2) & 3) ^ 3) +  0] =
 	X[(((tailIndex - 2) & ~3) << 1) + (((tailIndex - 2) & 3) ^ 3) +  4] = p[(iters >>  6) & 0x3f];
-      X[(((tailIndex - 3) & ~3) << 1) + (((tailIndex - 3) & 3) ^ 3) +  0] =
+      }
+      if ( iters >> 12 ) { 
+	X[(((tailIndex - 3) & ~3) << 1) + (((tailIndex - 3) & 3) ^ 3) +  0] =
 	X[(((tailIndex - 3) & ~3) << 1) + (((tailIndex - 3) & 3) ^ 3) +  4] = p[(iters >> 12) & 0x3f];
-      X[(((tailIndex - 4) & ~3) << 1) + (((tailIndex - 4) & 3) ^ 3) +  0] =
+      }
+      if ( iters >> 18 ) { 
+	X[(((tailIndex - 4) & ~3) << 1) + (((tailIndex - 4) & 3) ^ 3) +  0] =
 	X[(((tailIndex - 4) & ~3) << 1) + (((tailIndex - 4) & 3) ^ 3) +  4] = p[(iters >> 18) & 0x3f];
-      X[(((tailIndex - 5) & ~3) << 1) + (((tailIndex - 5) & 3) ^ 3) +  0] =
+      }
+      if ( iters >> 24 ) {
+	X[(((tailIndex - 5) & ~3) << 1) + (((tailIndex - 5) & 3) ^ 3) +  0] =
 	X[(((tailIndex - 5) & ~3) << 1) + (((tailIndex - 5) & 3) ^ 3) +  4] = p[(iters >> 24) & 0x3f];
-      X[(((tailIndex - 6) & ~3) << 1) + (((tailIndex - 6) & 3) ^ 3) +  0] =
+      }
+      if ( iters >> 30 ) { 
+	X[(((tailIndex - 6) & ~3) << 1) + (((tailIndex - 6) & 3) ^ 3) +  0] =
 	X[(((tailIndex - 6) & ~3) << 1) + (((tailIndex - 6) & 3) ^ 3) +  4] = p[(iters >> 30) & 0x3f];
+      }
     }
 		
     /* Force compiler to flush and reload MMX registers */
