@@ -77,9 +77,7 @@
 #define PPUTS(f,str) PP { fputs(str,f); }
 #define VPUTS(f,str) VV { fputs(str,f); }
 
-#define HDR_LINE_LEN 70
-void printwrap( FILE* fp, const char* hdr, const char* str, char c, 
-		int line_len );
+#define HDR_LINE_LEN 80
 char *read_header( FILE* f, char** s, int* slen, int* alloc, 
 		   char* a, int alen );
 int read_eof( FILE* fp, char* a );
@@ -91,6 +89,12 @@ void usage( const char* );
 int parse_period( const char* aperiod, long* resp );
 double report_speed( int bits, double* time_est, int display );
 
+#if defined( WIN32 )
+#define LINEFEED "\r\n"
+#else
+#define LINEFEED "\n"
+#endif
+
 /* avoid needing to export stolower from hashcash.dll */
 
 void mystolower( char* str );
@@ -116,15 +120,18 @@ typedef struct {
 } ARRAY;
 
 void db_open( DB* db, const char* db_filename );
-void db_purge( DB* db, ARRAY* purge_resource, int purge_all, 
-	       long purge_period, time_t now_time, time_t real_time,
-	       long grace_period );
+void db_purge_arr( DB* db, ARRAY* purge_resource, int purge_all, 
+		   long purge_period, time_t now_time, long validity_period,
+		   long grace_period );
+int db_purge( DB* db, ARRAY* purge_resource, int purge_all, 
+	      long purge_period, time_t now_time, long validity_period,
+	      long grace_period, int verbose, int* err );
 int db_in( DB* db, char* token, char *period );
 void db_add( DB* db, char* token, char *token_utime );
 void db_close( DB* db ) ;
 
 void array_alloc( ARRAY* array, int num );
-void array_push( ARRAY* array, char *str, int type, int case_flag, 
+void array_push( ARRAY* array, const char *str, int type, int case_flag, 
 		 long validity, long grace, long anon, int width, int bits, 
 		 int over );
 #define array_num( a ) ( (a)->num )
@@ -163,7 +170,9 @@ int main( int argc, char* argv[] )
     int mint_flag = 0, ignore_boundary_flag = 0, name_flag = 0, res_flag = 0;
     int auto_version = 0, version_flag = 0, checked = 0, comma = 0;
     char header[ MAX_HDR+1 ] = { 0 };
-    int header_len, token_found = 0, headers_found = 0;
+    char header2[ MAX_HDR+1 ] = { 0 };
+    char* header_wrapped = NULL;
+    int hdr_len, hdr2_len, token_found = 0, token2_found = 0, hdrs_found = 0;
     char token[ MAX_TOK+1 ] = { 0 }, token_resource[ MAX_RES+1 ] = { 0 };
     char *new_token = NULL ;
     char line_arr[ MAX_LINE+1 ] = { 0 }, *line = line_arr;
@@ -183,9 +192,9 @@ int main( int argc, char* argv[] )
     time_t now_time = real_time;
     time_t token_time = 0, expiry_time = 0;
     int time_width_flag = 0;	/* -z option, default 6 YYMMDD */
-    int small_flag = 0;		/* fast by default */
+    int compress = 0;		/* fast by default */
     int inferred_time_width = 0, time_width = 6; /* default YYMMDD */
-    int core = 0, res = 0;
+    int core = 0, res = 0, core_flag = 0;
 
     double tries_taken = 0, taken = 0, tries_expected = 0, time_est = 0;
     int opt = 0, vers = 0, db_opened = 0, i = 0, t = 0, tty_info = 0;
@@ -214,7 +223,7 @@ int main( int argc, char* argv[] )
     array_alloc( &args, 32 );
 
     while ( (opt=getopt(argc, argv, 
-		"-a:b:cde:f:g:hij:klmnop:qr:sSt:uvwx:yz:CEMO:PSVXZ")) >0 ) {
+		"-a:b:cde:f:g:hij:klmnop:qr:st:uvwx:yz:CEMO:PSVXZ:")) >0 ) {
 	switch ( opt ) {
 	case 'a': anon_flag = 1; 
 	    if ( !parse_period( optarg, &anon_period ) ) {
@@ -242,8 +251,7 @@ int main( int argc, char* argv[] )
 	    }
 	    break;
 	case 'C': case_flag = 1; break;
-	case 'c': check_flag = 1; 
-	    break;
+	case 'c': check_flag = 1; break;
 	case 'd': db_flag = 1; break;
 	case 'e': 
 	    if ( validity_flag ) { multiple_validity = 1; }
@@ -253,9 +261,7 @@ int main( int argc, char* argv[] )
 		usage( "error: -e invalid validity period" ); 
 	    }
 	    break;
-	case 'E': 
-	    str_type = TYPE_REGEXP; 
-	    break;
+	case 'E': str_type = TYPE_REGEXP; break;
 	case 'f': db_filename = strdup( optarg ); break;
 	case 'g': 
 	    if ( grace_flag ) { multiple_grace = 1; }
@@ -278,7 +284,9 @@ int main( int argc, char* argv[] )
 	case 'l': left_flag = 1; break;
 	case 'n': name_flag = 1; break;
 	case 'o': over_flag = 1; break;
-	case 'O': core = atoi( optarg ); 
+	case 'O': 
+	    core_flag = 1; 
+	    core = atoi( optarg ); 
 	    res = hashcash_use_core( core );
 	    if ( res < 1 ) {
 		usage( res == -1 ? "error: -O no such core\n" : 
@@ -291,9 +299,7 @@ int main( int argc, char* argv[] )
 		bits = default_bits;
 	    }
 	    break;
-	case 'M':
-	    str_type = TYPE_WILD;
-	    break;
+	case 'M': str_type = TYPE_WILD; break;
 	case 'p': 
 	    if ( purge_flag ) { usage("error: only one -p flag per call"); }
 	    purge_flag = 1;
@@ -365,6 +371,7 @@ int main( int argc, char* argv[] )
 	case 'X':
 	    hdr_flag = 1;
 	    sstrncpy( header, "X-Hashcash: ", MAX_HDR );
+	    sstrncpy( header2, "Hashcash: ", MAX_HDR );
 	    break;
 	case 'y': yes_flag = 1; break;
 	case 'z': 
@@ -374,7 +381,10 @@ int main( int argc, char* argv[] )
 	        usage( "error: -z invalid time width: must be 6, 10 or 12" );
 	    }
 	    break;
-	case 'Z': small_flag = 1; break;
+	case 'Z': compress = atoi( optarg ); 
+            /* temporary work around for bug discovered in 1.15, disable -Z2 */
+/*	    if ( compress > 1 ) { compress = 1; } */
+	    break;
 	case '?': 
 	    fprintf( stderr, "error: unrecognized option -%c", optopt );
 	    usage( "" );
@@ -424,14 +434,6 @@ int main( int argc, char* argv[] )
 	usage( "must specify at least one of -m, -c, -d, -n, -l-, -w, -b, -r, -p or -s");
     }
 
-//    if ( check_flag && hdr_flag && name_flag + left_flag + width_flag > 0 ) {
-//	usage( "must not specify -n, -l, or -w with -cx or -cX" );
-//    }
-
-//    if ( check_flag && hdr_flag && !res_flag ) {
-//	usage( "must give -r resource with -cx or -cX" );
-//    }
-
     if ( quiet_flag ) {	verbose_flag = 0; } /* quiet overrides verbose */
     if ( speed_flag && check_flag ) { speed_flag = 0; }	/* ignore speed */
 
@@ -439,7 +441,7 @@ int main( int argc, char* argv[] )
 	db_open( &db, db_filename );
 	db_opened = 1;
 
-	db_purge( &db, &purge_resource, purge_all, purge_period, 
+	db_purge_arr( &db, &purge_resource, purge_all, purge_period, 
 		  now_time, validity_flag ? purge_validity_period : 0,
 		  grace_period );
 
@@ -476,7 +478,11 @@ int main( int argc, char* argv[] )
 	/* integrate the bench test */
 
 	if ( verbose_flag && speed_flag && !mint_flag ) {
-	    hashcash_benchtest( 3 );
+	    if ( core_flag ) {
+		hashcash_benchtest( 3, core );
+	    } else {
+		hashcash_benchtest( 3, -1 );
+	    }
 	    exit( EXIT_SUCCESS ); /* don't actually calculate it */
 	}
 
@@ -486,6 +492,7 @@ int main( int argc, char* argv[] )
 	    QPRINTF( stderr, "speed: %ld collision tests per second\n",
                      hashcash_per_sec() );
             PPRINTF( stdout, "%ld\n", hashcash_per_sec() );
+	    QPRINTF( stderr, "compression: %d\n", compress );
 	}
 
 	if ( verbose_flag || ( speed_flag && !bits_flag ) ) {
@@ -495,6 +502,9 @@ int main( int argc, char* argv[] )
                      hashcash_per_sec() );
 	    if ( speed_flag && !bits_flag && !mint_flag ) {
                 PPRINTF( stdout, "%ld\n", hashcash_per_sec() );
+	    }
+	    QPRINTF( stderr, "compression: %d\n", compress );
+	    if ( speed_flag && !bits_flag && !mint_flag ) {
 		exit( EXIT_SUCCESS ); /* don't actually calculate it */
 	    }
 	}
@@ -519,7 +529,7 @@ int main( int argc, char* argv[] )
 
 	    err = hashcash_mint( now_time, ent->width, ent->str, ent->bits, 
 				 ent->anon, &new_token, &anon_random, 
-				 &tries_taken, ext, small_flag, 
+				 &tries_taken, ext, compress, 
 				 callback, NULL );
 	    end = clock();
 
@@ -560,8 +570,17 @@ int main( int argc, char* argv[] )
 	    taken = (end-start)/(double)CLOCKS_PER_SEC;
 	    VPRINTF( stderr, "time: %.0f seconds\n", taken );
 	    
-	    if ( hdr_flag ) { 
-		printwrap( stdout, header, new_token, '\t', HDR_LINE_LEN ); 
+	    if ( hdr_flag ) {
+		header_wrapped = hashcash_make_header( new_token, HDR_LINE_LEN,
+						       header, '\t', 
+						       LINEFEED );
+		if ( header_wrapped == NULL ) { 
+		    /* don't expect this but for security fail closed */
+		    fprintf(stderr, "out of memory\n");
+		    exit( EXIT_FAILURE );
+		}
+		fprintf( stdout, "%s", header_wrapped );
+		free( header_wrapped );
 	    } else { 
 		fprintf( stdout, "%s%s\n", 
 			 ((quiet_flag||!out_is_tty)? "" : "hashcash token: "),
@@ -575,8 +594,13 @@ int main( int argc, char* argv[] )
 		left_flag || bits_flag || res_flag || db_flag ) {
 
 	/* check token is valid */
-	trimspace(header);	/* be forgiving about missing space */
-	header_len = strlen( header );
+	trimspace( header );	/* be forgiving about missing space */
+	hdr_len = strlen( header );
+
+	if ( header2[0] ) {
+	    trimspace( header2 );
+	    hdr2_len = strlen( header2 );
+	}
 
 	/* if no resources given create a kind of NULL entry to carry */
 	/* the other options along; hashcash_check knows to not check */
@@ -587,7 +611,7 @@ int main( int argc, char* argv[] )
 			grace_period, anon_period, time_width, bits, 0 );
 	}
 
-	headers_found = 0;
+	hdrs_found = 0;
 	tty_info = 0;
 	in_headers = 0;
 
@@ -622,10 +646,16 @@ int main( int argc, char* argv[] )
 		if ( line[0] == '\0' ) { continue; }
 		if ( hdr_flag )	{
 		    token_found = (strncasecmp( line, header, 
-						header_len ) == 0);
+						hdr_len ) == 0);
+		    token2_found = header2[0] && 
+			(strncasecmp( line, header2, hdr2_len ) == 0);
 		    if ( token_found ) { 
-			sstrncpy( token, line+header_len, MAX_TOK );
-			headers_found = 1; 
+			sstrncpy( token, line+hdr_len, MAX_TOK );
+			hdrs_found = 1; 
+		    } else if ( token2_found ) { 
+			sstrncpy( token, line+hdr2_len, MAX_TOK );
+			hdrs_found = 1;
+			token_found = 1;
 		    }
 		} else {
 		    token_found = 1; 
@@ -861,7 +891,7 @@ int main( int argc, char* argv[] )
 			       "database: not double spent\n" );
 			checked = yes_flag || ( res_flag && bits_flag);
 			if ( checked ) {
-			    sprintf( period, "%ld", (long)validity_period );
+			    sprintf( period, "%ld", validity_period );
 			    db_add( &db, token, period );
 			}
 		    } else {
@@ -874,7 +904,7 @@ int main( int argc, char* argv[] )
 	    }
 	}
 	if ( !width_flag && !left_flag && !name_flag ) {
-	    if ( hdr_flag && !headers_found ) {
+	    if ( hdr_flag && !hdrs_found ) {
 		QPRINTF( stderr, "warning: no line matching %s found in input\n",
 			 header );
 	    } 
@@ -1013,12 +1043,14 @@ void usage( const char* msg )
     fprintf( stderr, "\t-W\t\tmatch following resources with wildcards (default)\n" );
     fprintf( stderr, "\t-E\t\tmatch following resources as regular expression\n" );
     fprintf( stderr, "\t-P\t\tshow progress while searching\n");
-    fprintf( stderr, "\t-O core\tuse specified minting core\n");
+    fprintf( stderr, "\t-O core\t\tuse specified minting core\n");
+    fprintf( stderr, "\t-Z n\t\t0 = fast (default), 1 = medium, 2 = small/slow\n");
     fprintf( stderr, "examples:\n" );
     fprintf( stderr, "\thashcash -mb20 foo                               # mint 20 bit collision\n" );
     fprintf( stderr, "\thashcash -cdb20 -r foo 1:20:040806:foo::831d0c6f22eb81ff:15eae4 # check collision\n" );
     fprintf( stderr, "\n" );
     fprintf( stderr, "see hashcash (1) man page or http://www.hashcash.org/ for more details.\n" );
+    fflush( stderr );
     exit( EXIT_ERROR );
 }
 
@@ -1104,19 +1136,45 @@ static int sdb_cb_token_matcher( const char* key, char* val,
     return 1;			/* otherwise keep */
 }
 
-void db_purge( DB* db, ARRAY* purge_resource, int purge_all, 
+int hashcash_db_purge( DB* db, const char* purge_resource, int type,
+		       int case_flag, long validity_period, long grace_period,
+		       int purge_all, long purge_period, time_t now_time,
+		       int* err ) {
+    ARRAY* purge_resource_arr = NULL;
+    array_alloc( purge_resource_arr, 1 );
+    array_push( purge_resource_arr, purge_resource, type, case_flag,
+		validity_period, grace_period, 0, 0, 0, 0 );
+    return db_purge( db, purge_resource_arr, purge_all, purge_period, 
+		     now_time, validity_period, grace_period, 0, err );
+}
+
+void db_purge_arr( DB* db, ARRAY* purge_resource, int purge_all, 
 	       long purge_period, time_t now_time, long validity_period,
 	       long grace_period ) {
-    int err = 0 ;
+    int res, err = HASHCASH_FAIL;
+    res = db_purge( db, purge_resource, purge_all, purge_period, now_time, 
+		    validity_period, grace_period, verbose_flag, &err );
+    switch ( res ) {
+    case HASHCASH_INVALID_TIME:
+	die_msg( "error: invalid time argument" ); break;
+    case HASHCASH_FAIL: die( err ); break;
+    case HASHCASH_OK: break;
+    default: die( err );
+    }
+}
+
+int db_purge( DB* db, ARRAY* purge_resource, int purge_all, 
+	       long purge_period, time_t now_time, long validity_period,
+	       long grace_period, int verbose, int* err ) {
     time_t last_time = 0 ;
     char purge_utime[ MAX_UTC+1 ] = {0}; /* time token created */
     int ret = 0;
     db_arg arg;
 
-    if ( now_time < 0 ) { die_msg( "error: outside unix time Epoch" ); }
+    if ( now_time < 0 ) { return HASHCASH_INVALID_TIME; }
 
-    if ( !sdb_lookup( db, PURGED_KEY, purge_utime, MAX_UTC, &err ) ) {
-	die( err );
+    if ( !sdb_lookup( db, PURGED_KEY, purge_utime, MAX_UTC, err ) ) {
+	return 0;
     }
 
     last_time = hashcash_from_utctimestr( purge_utime, 1 );
@@ -1125,7 +1183,7 @@ void db_purge( DB* db, ARRAY* purge_resource, int purge_all,
     }
 
     if ( !hashcash_to_utctimestr( arg.now_utime, MAX_UTC, now_time ) ) {
-	die_msg( "error: converting time" );
+	return HASHCASH_INVALID_TIME;
     }
 
     arg.expires_before = now_time;
@@ -1136,41 +1194,40 @@ void db_purge( DB* db, ARRAY* purge_resource, int purge_all,
 
     if ( purge_period == 0 || now_time >= last_time + purge_period ) {
 	VPRINTF( stderr, "purging database: ..." );
-	ret = sdb_updateiterate( db, sdb_cb_token_matcher, (void*)&arg, &err );
+	ret = sdb_updateiterate( db, sdb_cb_token_matcher, (void*)&arg, err );
 	VPRINTF( stderr, ret ? "done\n" : "failed\n" ); 
+    } else {
+	ret = 1;
     }
+    return ret;
 }
 
 void db_open( DB* db, const char* db_filename ) {
-    int err = 0 ;
-
-    if ( !sdb_open( db, db_filename, &err ) ) { die( err ); }
-    fgetc( db->file );		/* try read to trigger EOF */
-    if ( feof( db->file ) ) {
-	if ( !sdb_add( db, PURGED_KEY, "700101000000", &err ) ) { 
-	    die( err ); 
-	}
-    }
-    rewind( db->file );
+    int err;
+    if (!hashcash_db_open( db, db_filename, &err )) { die(err); }
 }
 
 int db_in( DB* db, char* token, char *period ) {
-    int err = 0 ;
-    int in_db = 0 ;
+    int err = 0;
+    int res;
 
-    in_db = sdb_lookup( db, token, period, MAX_UTC, &err ); 
+    res = hashcash_db_in( db, token, period, &err );
     if ( err ) { die( err ); }
-    return in_db;
+    return res;
 }
 
 void db_add( DB* db, char* token, char *period ) {
-    int err = 0 ;
-    if ( !sdb_add( db, token, period, &err ) ) { die( err ); }
+    int err = 0;
+    if ( !hashcash_db_add( db, token, period, &err ) ) {
+	die( err );
+    }
 }
 
 void db_close( DB* db ) {
-    int err = 0 ;
-    if ( !sdb_close( db, &err ) ) { die( err ); }
+    int err = 0;
+    if (!hashcash_db_close( db, &err ) ) {
+	die( err );
+    }
 }
 
 /* num = start size, auto-grows */ 
@@ -1184,7 +1241,7 @@ void array_alloc( ARRAY* array, int num ) {
 
 /* auto-grow array */
 
-void array_push( ARRAY* array, char *str, int type, int case_flag, 
+void array_push( ARRAY* array, const char *str, int type, int case_flag, 
 		 long validity, long grace, long anon, int width, int bits, 
 		 int over ) 
 {
@@ -1252,46 +1309,6 @@ void die_msg( const char* str )
     exit( EXIT_ERROR );
 }
 
-void printwrap( FILE* fp, const char* hdr, const char* str, char c, 
-		int line_len )
-{
-    int str_len = strlen( str ), i, fstep, step;
-    char fformat[20], format[20];
-    char line_arr[MAX_LINE+1];
-    char* line = line_arr;
-    int line_alloc = 0;
-
-    if ( line_len > MAX_LINE ) { line = malloc(line_len+1); line_alloc = 1; }
-
-    if ( c == '\\' ) {
-	fstep = line_len - strlen(hdr) - 1;
-	step = line_len - 1;
-	sprintf( fformat, "%s%%s\\\n", hdr );
-	sprintf( format, "%%s\\\n" );
-    } else if ( c == '\t' ) {
-	fstep = line_len - strlen(hdr);
-	step = line_len - 8;
-	sprintf( fformat, "%s%%s\n", hdr );
-	sprintf( format, "\t%%s\n" );
-    } else if ( c == 0 ) {
-	fstep = line_len - strlen(hdr);
-	step = line_len;
-	sprintf( fformat, "%s%%s\n", hdr );
-	sprintf( format, "%%s\n" );
-    } else {
-	fstep = line_len - strlen(hdr);
-	step = line_len-1;
-	sprintf( fformat, "%s%%s\n", hdr );
-	sprintf( format, "%c%%s\n", c );
-    }
-    for ( i = 0; i < str_len; 
-	  str += i ? step : fstep, i += i ? step : fstep ) {
-	sstrncpy( line, str, i ? step : fstep );
-	fprintf( fp, i ? format : fformat, line );
-    }
-    if ( line_alloc ) { free( line ); }
-}
-
 /* remove unix, DOS and MAC linefeeds from line ending */
 
 void chomplf( char* token ) {
@@ -1396,10 +1413,10 @@ char *read_header( FILE* f, char** s, int* smax, int* alloc,
 	a[0] = '\0';
 	fgets( a, amax, f );
 	chomplf( a );
-	if ( a[0] == '\t' ) {
+	if ( a[0] == '\t' || a[0] == ' ') {
 	    read_append( s, smax, alloc, a+1 );
 	}
-    } while ( a[0] == '\t' );
+    } while ( a[0] == '\t' || a[0] == ' ' );
 
     return *s;
 }
@@ -1415,4 +1432,3 @@ void mystolower( char* str ) {
         *str = tolower( *str );
     }
 }
-
